## tpad version 1.3
# WinXP (TM) enhanced Notepad clone written in Tcl/Tk.
# ASCII table plugin
# Copyright (C) 2004  Antonio Bonifati
#
# Please see the main file tpad.tcl for licence and disclaimer
#
## last revised on: $Date: 2004/08/31 $

package provide tpad 1.3

# make sure the msgcat namespace exists
namespace eval ::msgcat {}

namespace eval ::tpad::ascii {
  namespace export display

  namespace import ::msgcat::mc ::msgcat::mcmax
}

proc ::tpad::ascii::Press {pn chs x y} {
  variable ::tpad::$pn\::p
  variable keypressed

  # find out pressed key in the ascii table and highlight it in inverse colors
  set keypressed [expr $x / $chs * 16 + $y / $chs]
  set m $p.ascii
  $m.table itemconfigure text$keypressed -fill $::tpad::bg
  $m.table itemconfigure rect$keypressed -fill $::tpad::fg
}

proc ::tpad::ascii::Release {pn chs x y} {
  variable ::tpad::$pn\::p
  variable keypressed

  # find out on what key the mouse was released on the ascii table
  set keyreleased [expr $x / $chs * 16 + $y / $chs]
  if {$keyreleased==$keypressed} {
     # the mouse was released just under the key were it was pressed

     # skip the 127th char
     # this avoids calls to the Tk font locate routines which are slow
     if {$keypressed!=127} {
       eval $p.text insert insert \\x[format %02x $keypressed]
       updateStatusBar $pn
     }
  }
  # restore pressed key to normal colors
  set m $p.ascii
  $m.table itemconfigure text$keypressed -fill $::tpad::fg
  $m.table itemconfigure rect$keypressed -fill $::tpad::bg
}

# a convenient ascii table to insert characters in a text
proc ::tpad::ascii::display {pn} {
  variable ::tpad::$pn\::p

  set m $p.ascii
  toplevel $m
  wm group $m $pn
  wm title $m [mc "ASCII table"]
  wm transient $m $pn
  wm resizable $m no no
  # catch the kill of the window manager
  wm protocol $m WM_DELETE_WINDOW [namespace code "
    set ::tpad::$pn\::ascii no
    destroy $m
  "]

  # estimate maximum character size and figure out a cell size from it
  set chs [expr round([font metrics $::tpad::ascii::font -linespace] * 1.5)]
  set hchs [expr $chs / 2]  ;# half of chs
  set size [expr $chs * 16 + 1] ;# canvas size

  # display a 16x16 rectangular grid made up with rectangles
  # and fill the grid with the ASCII and Latin-1 character sets
  canvas $m.table -width $size -height $size -bg $::tpad::bg
  for {set i 0; set x 1} {$x < $size} {incr x $chs} {
    for {set y 1} {$y < $size} {incr y $chs; incr i} {
      eval set chr \\x[format %02x $i]

      $m.table create rect $x $y [expr $x + $chs] [expr $y + $chs] \
        -outline $::tpad::fg -tags rect$i

      # skip missing glyphs, usually control and non-Latin chars,
      # except for some control chars which are represented by
      # well-known C/Tcl escapes

      if {$i<=32} {
        switch $i {
          7 {set chr \\a}
          8 {set chr \\b}
          9 {set chr \\t}
          10 {set chr \\n}
          11 {set chr \\v}
          12 {set chr \\f}
          13 {set chr \\r}
          default {set chr { }}
        }
      }

      if {$chr!={ } && !($i>=127 && $i<=159)} {
        $m.table create text [expr $x + $hchs] [expr $y + $hchs] \
          -text $chr -font $::tpad::ascii::font -fill $::tpad::fg -tags text$i
      }
    }
  }

  pack $m.table

  bind $m.table <ButtonPress> [namespace code "Press $pn $chs %x %y"]
  bind $m.table <ButtonRelease> [
    namespace code "Release $pn $chs %x %y"
  ]
  bind $m <Key-Escape>  [namespace code "
    set ::tpad::$pn\::ascii no
    destroy $m
  "]

  ::tpad::PlaceWindow $pn $m
}
